@@ -121,7 +121,6 @@ MInstall::~MInstall() {
     if (oobe) delete oobe;
     if (base) delete base;
     if (bootman) delete bootman;
-    if (swapman) delete swapman;
     if (partman) delete partman;
     if (throbber) delete throbber;
 }
@@ -130,7 +129,6 @@ MInstall::~MInstall() {
 void MInstall::startup()
 {
     proc.log(__PRETTY_FUNCTION__, MProcess::Section);
-    connect(pushClose, &QPushButton::clicked, this, &MInstall::close);
 
     if (!modeOOBE) {
         // Check for a bad combination, like 32-bit ISO and 64-bit UEFI.
@@ -147,10 +145,9 @@ void MInstall::startup()
         base = new Base(proc, *partman, *this, appConf, appArgs);
         base->scanMedia();
         bootman = new BootMan(proc, *partman, *this, appConf, appArgs);
-        swapman = new SwapMan(proc, *partman, *this);
-        autopart = new AutoPart(proc, partman, *this, appConf, 16*MB);
-        partman->autopart = autopart;
 
+        ROOT_BUFFER = appConf.value("ROOT_BUFFER", 5000).toInt();
+        HOME_BUFFER = appConf.value("HOME_BUFFER", 2000).toInt();
         INSTALL_FROM_ROOT_DEVICE = appConf.value("INSTALL_FROM_ROOT_DEVICE").toBool();
 
         // Unable to get space requirements since there's no media.
@@ -178,15 +175,12 @@ void MInstall::startup()
     // Password box setup
     textCryptoPass->setup(textCryptoPass2, progCryptoPassMeter, 1, 32, 9);
     textCryptoPassCust->setup(textCryptoPassCust2, progCryptoPassMeterCust, 1, 32, 9);
-    connect(textCryptoPass, &MPassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
-    connect(textCryptoPassCust, &MPassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
+    connect(textCryptoPass, &MPassEdit::validationChanged, this, &MInstall::diskPassValidationChanged);
+    connect(textCryptoPassCust, &MPassEdit::validationChanged, this, &MInstall::diskPassValidationChanged);
 
     setupkeyboardbutton();
 
-    // Guess if hardware clock is UTC or local time
-    proc.shell("guess-hwclock", nullptr, true);
-    if (proc.readOut() == "localtime") checkLocalClock->setChecked(true);
-    else if (proc.detectMac()) checkLocalClock->setChecked(true);
+    if (proc.detectMac()) checkLocalClock->setChecked(true);
 
     oobe = new Oobe(proc, *this, this, appConf, oem, modeOOBE);
 
@@ -201,7 +195,6 @@ void MInstall::startup()
                 item->addToCombo(comboDisk);
             }
         }
-        connect(radioEntireDisk, &QCheckBox::toggled, boxAutoPart, &QGroupBox::setEnabled);
         if (comboDisk->count() >= 1) {
             comboDisk->setCurrentIndex(0);
             radioEntireDisk->setChecked(true);
@@ -217,12 +210,9 @@ void MInstall::startup()
             boxAutoPart->setEnabled(false);
             radioCustomPart->setChecked(true);
         }
-        autopart->refresh();
+        setupPartitionSlider();
         // Override with whatever is in the config.
         manageConfig(ConfigLoadA);
-        // Hibernation check box (regular install).
-        checkHibernationReg->setChecked(checkHibernation->isChecked());
-        connect(checkHibernationReg, &QCheckBox::clicked, checkHibernation, &QCheckBox::setChecked);
     }
     oobe->stashServices(true);
 
@@ -393,8 +383,12 @@ bool MInstall::processNextPhase()
 
             // the core of the installation
             if (!pretend) {
-                proc.advance(11, partman->countPrepSteps());
-                partman->prepStorage();
+                proc.advance(12, partman->countPrepSteps());
+                partman->preparePartitions();
+                partman->formatPartitions();
+                partman->mountPartitions();
+                //run blkid -c /dev/null to freshen UUID cache
+                proc.exec("blkid", {"-c", "/dev/null"});
                 base->install();
             } else {
                 if (!pretendToInstall(14, 200)) return false;
@@ -419,7 +413,6 @@ bool MInstall::processNextPhase()
                 manageConfig(ConfigSave);
                 config->dumpDebug();
                 proc.exec("/bin/sync"); // the sync(2) system call will block the GUI
-                swapman->install();
                 bootman->install();
                 if (oem) proc.shell("sed -i 's/splash\\b/nosplash/g' /mnt/antiX/boot/grub/grub.cfg");
             } else if (!pretendToInstall(5, 100)) {
@@ -478,10 +471,11 @@ void MInstall::manageConfig(enum ConfigAction mode)
             if (mode == ConfigSave) {
                 config->setValue("RootPortion", sliderPart->value());
             } else if (config->contains("RootPortion")) {
-                 const int portion = config->value("RootPortion").toInt();
-                 sliderPart->setSliderPosition(portion);
-                 if (sliderPart->value() != portion) {
-                     config->markBadWidget(boxSliderPart);
+                 const int sliderVal = config->value("RootPortion").toInt();
+                 sliderPart->setValue(sliderVal);
+                 on_sliderPart_valueChanged(sliderVal);
+                 if (sliderPart->value() != sliderVal) {
+                     config->markBadWidget(sliderPart);
                  }
             }
         }
@@ -508,10 +502,10 @@ void MInstall::manageConfig(enum ConfigAction mode)
         config->endGroup();
     }
 
-    const bool confAdvanced = (!modeOOBE && radioCustomPart->isChecked());
-    if (confAdvanced) swapman->manageConfig(*config);
     if (mode == ConfigSave || mode == ConfigLoadB) {
-        if (confAdvanced) bootman->manageConfig(*config);
+        // GRUB page
+        if (!modeOOBE && radioCustomPart->isChecked()) bootman->manageConfig(*config);
+        // Manage the rest of the OOBE pages.
         oobe->manageConfig(*config, mode==ConfigSave);
     }
 
@@ -568,7 +562,7 @@ int MInstall::showPage(int curr, int next)
                 QString msg = tr("OK to format and use the entire disk (%1) for %2?");
                 if (!proc.detectEFI()) {
                     DeviceItem *devit = partman->findByPath("/dev/" + comboDisk->currentData().toString());
-                    if (devit && devit->size >= 2*TB) {
+                    if (devit && devit->size >= (2048LL*1073741824LL)) {
                         msg += "\n\n" + tr("WARNING: The selected drive has a capacity of at least 2TB and must be formatted using GPT."
                                            " On some systems, a GPT-formatted disk will not boot.");
                         return curr;
@@ -580,16 +574,14 @@ int MInstall::showPage(int curr, int next)
                 if (ans != QMessageBox::Yes) return curr; // don't format - stop install
             }
             partman->clearAllUses();
-            autopart->buildLayout(autopart->partSize(), boxEncryptAuto->isChecked());
+            partman->selectedDriveAuto()->layoutDefault(-1, boxEncryptAuto->isChecked());
             if (!partman->composeValidate(true, PROJECTNAME)) {
                 nextFocus = treePartitions;
                 return curr;
             }
             bootman->buildBootLists(); // Load default boot options
-            swapman->setupDefaults();
             manageConfig(ConfigLoadB);
-            checkHibernation->setChecked(checkHibernationReg->isChecked());
-            return Step::Network;
+            return oem ? Step::Progress : Step::Network;
         }
     } else if (curr == Step::Partitions && next > curr) {
         if (!partman->composeValidate(automatic, PROJECTNAME)) {
@@ -604,8 +596,16 @@ int MInstall::showPage(int curr, int next)
         }
         return Step::Boot;
     } else if (curr == Step::Boot && next > curr) {
-        if (oem) return Step::Progress;
-        return Step::Services + 1;
+        return oem ? Step::Progress : Step::Network;
+    } else if (curr == Step::Network && next > curr) {
+        nextFocus = oobe->validateComputerName();
+        if (nextFocus) return curr;
+    } else if (curr == Step::Network && next < curr) { // Backward
+        return Step::Boot; // Skip pageServices
+    } else if (curr == Step::Localization && next > curr) {
+        if (!pretend && oobe->haveSnapshotUserAccounts) {
+            return Step::Progress; // Skip pageUserAccounts and pageOldHome
+        }
     } else if (curr == Step::UserAccounts && next > curr) {
         nextFocus = oobe->validateUserInfo(automatic);
         if (nextFocus) return curr;
@@ -616,15 +616,6 @@ int MInstall::showPage(int curr, int next)
             const QString &str = tr("The home directory for %1 already exists.");
             labelOldHome->setText(str.arg(textUserName->text()));
         }
-    } else if (curr == Step::Network && next > curr) {
-        nextFocus = oobe->validateComputerName();
-        if (nextFocus) return curr;
-    } else if (curr == Step::Network && next < curr) { // Backward
-        return Step::Boot; // Skip pageServices
-    } else if (curr == Step::Localization && next > curr) {
-        if (!pretend && oobe->haveSnapshotUserAccounts) {
-            return Step::Progress; // Skip pageUserAccounts and pageOldHome
-        }
     } else if (curr == Step::OldHome && next < curr) { // Backward
         if (!pretend && oobe->haveSnapshotUserAccounts) {
             return Step::Localization; // Skip pageUserAccounts and pageOldHome
@@ -656,6 +647,8 @@ void MInstall::pageDisplayed(int next)
         if (next != ixProgress) ixTipStart = ixTip;
     }
 
+    QString tminroot, trecroot; // Only used for Step::Disk
+
     switch (next) {
     case Step::Terms:
         textHelp->setText("<p><b>" + tr("General Instructions") + "</b><br/>"
@@ -668,8 +661,17 @@ void MInstall::pageDisplayed(int next)
         pushNext->setDefault(true);
         break;
     case Step::Disk:
+        if (partman) {
+            // These calculations are only for display text, and do not affect the installation.
+            long long rootMin = partman->rootSpaceNeeded + 1048575;
+            tminroot = QLocale::system().formattedDataSize(rootMin, 0, QLocale::DataSizeTraditionalFormat);
+            rootMin = (4 * rootMin) + ROOT_BUFFER * 1048576; // (Root + snapshot [squashfs + ISO] + backup) + buffer.
+            trecroot = QLocale::system().formattedDataSize(rootMin, 0, QLocale::DataSizeTraditionalFormat);
+        }
+
         textHelp->setText("<p><b>" + tr("Installation Options") + "</b><br/>"
-            + tr("If you are running Mac OS or Windows OS (from Vista onwards), you may have to use that system's software to set up partitions and boot manager before installing.") + "</p>"
+            + tr("Installation requires about %1 of space. %2 or more is preferred.").arg(tminroot, trecroot) + "</p>"
+            "<p>" + tr("If you are running Mac OS or Windows OS (from Vista onwards), you may have to use that system's software to set up partitions and boot manager before installing.") + "</p>"
             "<p><b>" + tr("Using the root-home space slider") + "</b><br/>"
             + tr("The drive can be divided into separate system (root) and user data (home) partitions using the slider.") + "</p>"
             "<p>" + tr("The <b>root</b> partition will contain the operating system and applications.") + "<br/>"
@@ -782,20 +784,14 @@ void MInstall::pageDisplayed(int next)
         break;
 
     case Step::Boot: // Start of installation.
-        textHelp->setText("<p><b>" + tr("Install GRUB for Linux and Windows") + "</b><br/>"
-            + tr("%1 uses the GRUB bootloader to boot %1 and Microsoft Windows.").arg(PROJECTNAME) + "</p>"
-            "<p>" + tr("By default GRUB is installed in the Master Boot Record (MBR) or ESP (EFI System Partition for 64-bit UEFI boot systems) of your boot drive and replaces the boot loader you were using before. This is normal.") + "</p>"
-            "<p>" + tr("If you choose to install GRUB to Partition Boot Record (PBR) instead, then GRUB will be installed at the beginning of the specified partition. This option is for experts only.") + "</p>"
-            "<p>" + tr("If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.") + "</p>"
-            "<p><b>" + tr("Create a swap file") + "</b><br/>"
-            + tr("A swap file is more flexible than a swap partition; it is considerably easier to resize a swap file to adapt to changes in system usage.") + "</p>"
-            "<p>" + tr("By default, this is checked if no swap partitions have been set, and unchecked if swap partitions are set. This option should be left untouched, and is for experts only.") + "<br/>"
-            + tr("Setting the size to 0 has the same effect as unchecking this option.") + "</p>");
+        textHelp->setText(tr("<p><b>Select Boot Method</b><br/> %1 uses the GRUB bootloader to boot %1 and MS-Windows. "
+                             "<p>By default GRUB2 is installed in the Master Boot Record (MBR) or ESP (EFI System Partition for 64-bit UEFI boot systems) of your boot drive and replaces the boot loader you were using before. This is normal.</p>"
+                             "<p>If you choose to install GRUB2 to Partition Boot Record (PBR) instead, then GRUB2 will be installed at the beginning of the specified partition. This option is for experts only.</p>"
+                             "<p>If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.</p>").arg(PROJECTNAME));
 
         enableBack = false;
         if (phase <= 0) {
             bootman->buildBootLists();
-            swapman->setupDefaults();
             manageConfig(ConfigLoadB);
         }
         break;
@@ -844,6 +840,7 @@ void MInstall::pageDisplayed(int next)
             " does not need to be secure, such as a public terminal.") + "</p>");
         if (!nextFocus) nextFocus = textUserName;
         oobe->userPassValidationChanged();
+        enableNext = false;
         break;
 
     case Step::OldHome:
@@ -886,6 +883,7 @@ void MInstall::pageDisplayed(int next)
             + "</p><p>"
             + tr("Complete these steps at your own pace. The installer will wait for your input if necessary.")
             + "</p>");
+        enableBack = !oem || radioCustomPart->isChecked();
         enableNext = false;
         break;
 
@@ -900,11 +898,11 @@ void MInstall::pageDisplayed(int next)
         break;
 
     default: // other
-        textHelp->setText("<p><b>" + tr("Enjoy using %1").arg(PROJECTNAME) + "</b></p>"
+        textHelp->setText("<p><b>" + tr("Enjoy using %1</b></p>").arg(PROJECTNAME) + "\n\n "
         + tr("<p><b>Support %1</b><br/>"
             "%1 is supported by people like you. Some help others at the "
             "support forum - %2 - or translate help files into different "
-            "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME, PROJECTFORUM));
+            "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
         pushNext->setDefault(true);
         break;
     }
@@ -996,6 +994,21 @@ void MInstall::gotoPage(int next)
     }
 }
 
+void MInstall::setupPartitionSlider()
+{
+    // Allow the slider labels to fit all possible formatted sizes.
+    const QString &strMB = sliderSizeString(1072693248) + '\n'; // "1,023 GB"
+    const QFontMetrics &fmetrics = labelSliderRoot->fontMetrics();
+    int mwidth = fmetrics.boundingRect(QRect(), Qt::AlignCenter, strMB + tr("Root")).width();
+    labelSliderRoot->setMinimumWidth(mwidth);
+    mwidth = fmetrics.boundingRect(QRect(), Qt::AlignCenter, strMB + tr("Home")).width();
+    labelSliderHome->setMinimumWidth(mwidth);
+    labelSliderRoot->setText("----");
+    labelSliderHome->setText("----");
+    // Snap the slider to the legal range.
+    on_sliderPart_valueChanged(sliderPart->value());
+}
+
 /////////////////////////////////////////////////////////////////////////
 // event handlers
 
@@ -1050,7 +1063,7 @@ void MInstall::changeEvent(QEvent *event)
         col.setAlpha(200);
         pal.setColor(QPalette::Base, col);
         textHelp->setPalette(pal);
-        if (autopart) autopart->refresh();
+        setupPartitionSlider();
     }
 }
 
@@ -1086,6 +1099,11 @@ void MInstall::reject()
 /////////////////////////////////////////////////////////////////////////
 // slots
 
+void MInstall::diskPassValidationChanged(bool valid)
+{
+    pushNext->setEnabled(valid);
+}
+
 void MInstall::on_pushNext_clicked()
 {
     gotoPage(widgetStack->currentIndex() + 1);
@@ -1175,7 +1193,7 @@ void MInstall::on_progInstall_valueChanged(int value)
                              "Basic information about %1 is at %2.</p><p>"
                              "There are volunteers to help you at the %3 forum, %4</p>"
                              "<p>If you ask for help, please remember to describe your problem and your computer "
-                             "in some detail. Usually statements like 'it didn't work' are not helpful.</p>").arg(PROJECTNAME, PROJECTURL, PROJECTSHORTNAME, PROJECTFORUM));
+                             "in some detail. Usually statements like 'it didn't work' are not helpful.</p>").arg(PROJECTNAME).arg(PROJECTURL).arg(PROJECTSHORTNAME).arg(PROJECTFORUM));
         break;
 
     case 1:
@@ -1188,7 +1206,7 @@ void MInstall::on_progInstall_valueChanged(int value)
         textTips->setText(tr("<p><b>Support %1</b><br/>"
                              "%1 is supported by people like you. Some help others at the "
                              "support forum - %2 - or translate help files into different "
-                             "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME, PROJECTFORUM));
+                             "languages, or make suggestions, write documentation, or help test new software.</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
 
         break;
 
@@ -1202,17 +1220,22 @@ void MInstall::on_progInstall_valueChanged(int value)
 
     case 4:
         textTips->setText(tr("<p><b>Keep Your Copy of %1 up-to-date</b><br/>"
-                             "For more information and updates please visit</p><p> %2</p>").arg(PROJECTNAME, PROJECTFORUM));
+                             "For more information and updates please visit</p><p> %2</p>").arg(PROJECTNAME).arg(PROJECTFORUM));
         break;
 
     default:
         textTips->setText(tr("<p><b>Special Thanks</b><br/>Thanks to everyone who has chosen to support %1 with their time, money, suggestions, work, praise, ideas, promotion, and/or encouragement.</p>"
                              "<p>Without you there would be no %1.</p>"
-                             "<p>%2 Dev Team</p>").arg(PROJECTNAME, PROJECTSHORTNAME));
+                             "<p>%2 Dev Team</p>").arg(PROJECTNAME).arg(PROJECTSHORTNAME));
         break;
     }
 }
 
+void MInstall::on_pushClose_clicked()
+{
+    close();
+}
+
 void MInstall::setupkeyboardbutton()
 {
     QFile file("/etc/default/keyboard");
@@ -1247,8 +1270,67 @@ void MInstall::on_pushSetKeyboard_clicked()
 
 void MInstall::on_comboDisk_currentIndexChanged(int)
 {
-    DeviceItem *devit = partman->findByPath("/dev/" + comboDisk->currentData().toString());
-    autopart->setDrive(devit, true, boxEncryptAuto->isChecked(), checkHibernationReg->isChecked(), true);
+    on_sliderPart_valueChanged(sliderPart->value());
+}
+
+void MInstall::on_sliderPart_sliderPressed()
+{
+    QString tipText(tr("%1% root\n%2% home"));
+    const int val = sliderPart->value();
+    if (val==100) tipText = tr("Combined root and home");
+    else if (val<1) tipText = tipText.arg(">0", "<100");
+    else tipText = tipText.arg(val).arg(100-val);
+    sliderPart->setToolTip(tipText);
+    if (sliderPart->isSliderDown()) {
+        QToolTip::showText(QCursor::pos(), tipText, sliderPart);
+    }
+}
+void MInstall::on_sliderPart_valueChanged(int value)
+{
+    if (!partman) return;
+    const bool crypto = boxEncryptAuto->isChecked();
+    DeviceItem *drvitem = partman->selectedDriveAuto();
+    if (!drvitem) return;
+    long long available = drvitem->layoutDefault(100, crypto, false);
+    if (!available) return;
+    const long long roundUp = available - 1;
+    const long long rootMinMB = (partman->rootSpaceNeeded + 1048575) / 1048576;
+    const long long homeMinMB = 16; // 16MB for basic profile and setup files
+    const long long rootRecMB = rootMinMB + ROOT_BUFFER;
+    const long long homeRecMB = homeMinMB + HOME_BUFFER;
+    const int minPercent = ((rootMinMB * 100) + roundUp) / available;
+    const int maxPercent = 100 - (((homeMinMB * 100) + roundUp) / available);
+    const int recPercentMin = ((rootRecMB * 100) + roundUp) / available; // Recommended root size.
+    const int recPercentMax = 100 - (((homeRecMB * 100) + roundUp) / available); // Recommended minimum home.
+
+    const int origValue = value;
+    if (value < minPercent) value = minPercent;
+    else if (value > maxPercent) value = 100;
+    if (value != origValue) {
+        qApp->beep();
+        sliderPart->blockSignals(true);
+        sliderPart->setValue(value);
+        sliderPart->blockSignals(false);
+    }
+
+    const long long availRoot = drvitem->layoutDefault(value, crypto, false);
+    QString valstr = sliderSizeString(availRoot*1048576);
+    available -= availRoot;
+    labelSliderRoot->setText(valstr + "\n" + tr("Root"));
+
+    QPalette palRoot = QApplication::palette();
+    QPalette palHome = QApplication::palette();
+    if (value < recPercentMin) palRoot.setColor(QPalette::WindowText, Qt::red);
+    if (value==100) valstr = tr("----");
+    else {
+        valstr = sliderSizeString(available*1048576);
+        valstr += "\n" + tr("Home");
+        if (value > recPercentMax) palHome.setColor(QPalette::WindowText, Qt::red);
+    }
+    labelSliderHome->setText(valstr);
+    labelSliderRoot->setPalette(palRoot);
+    labelSliderHome->setPalette(palHome);
+    on_sliderPart_sliderPressed(); // For the tool tip.
 }
 
 void MInstall::on_boxEncryptAuto_toggled(bool checked)
@@ -1256,11 +1338,7 @@ void MInstall::on_boxEncryptAuto_toggled(bool checked)
     pushNext->setEnabled(!checked || textCryptoPass->isValid());
     radioBootPBR->setDisabled(checked);
     // Account for addition/removal of the boot partition.
-    on_comboDisk_currentIndexChanged(0);
-}
-void MInstall::on_checkHibernationReg_toggled(bool)
-{
-    on_comboDisk_currentIndexChanged(0);
+    on_sliderPart_valueChanged(sliderPart->value());
 }
 
 void MInstall::on_radioCustomPart_toggled(bool checked)
