@@ -28,7 +28,6 @@
 
 class MProcess : public QProcess
 {
-    Q_OBJECT
     int execount = 0;
     int sleepcount = 0;
     bool halting = false;
@@ -37,7 +36,6 @@ class MProcess : public QProcess
     QProgressBar *progBar = nullptr;
     int progSliceStart = 0, progSliceSpace = 0;
     long progSlicePos = 0, progSliceSteps = 0;
-    const char *exceptionInfo = nullptr;
     // System detection results
     QString testArch;
     int testEFI = -1;
@@ -68,14 +66,12 @@ public:
     void halt();
     void unhalt();
     bool halted() const { return halting; }
-    // User interface
     static QString joinCommand(const QString &program, const QStringList &arguments);
     QListWidgetItem *log(const QString &text, const enum LogType type = Standard);
     void log(QListWidgetItem *entry, const int status = 1);
     void status(const QString &text, long progress = -1);
     void status(long progress = -1);
     void advance(int space, long steps);
-    void setExceptionMode(const char *failInfo);
     // Common functions that are traditionally carried out by processes.
     void sleep(const int msec, const bool silent = false);
     bool mkpath(const QString &path);
