@@ -20,6 +20,7 @@
  *
  * This file is part of the gazelle-installer.
  ***************************************************************************/
+#include <sys/stat.h>
 #include <QDebug>
 #include <QMessageBox>
 #include <QFileInfo>
@@ -386,7 +387,7 @@ int Oobe::selectTimeZone(const QString &zone)
 
 void Oobe::setLocale()
 {
-    proc.log(__PRETTY_FUNCTION__);
+    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     //locale
     qDebug() << "Update locale";
@@ -426,6 +427,8 @@ void Oobe::setLocale()
         proc.shell("sed -i '/data0=/c\\data0=%l:%M' " + skelpath + "/.config/xfce4/panel/xfce4-orageclock-plugin-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%l:%M' /home/demo/.config/xfce4/panel/datetime-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%l:%M' " + skelpath + "/.config/xfce4/panel/datetime-1.rc");
+        proc.shell("sed -i 's/%H/%l/' /home/demo/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+        proc.shell("sed -i 's/%H/%l/' " + skelpath + "/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
 
         //mx kde
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=0' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
@@ -445,6 +448,9 @@ void Oobe::setLocale()
         proc.shell("sed -i '/data0=/c\\data0=%H:%M' " + skelpath + "/.config/xfce4/panel/xfce4-orageclock-plugin-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%H:%M' /home/demo/.config/xfce4/panel/datetime-1.rc");
         proc.shell("sed -i '/time_format=/c\\time_format=%H:%M' " + skelpath + "/.config/xfce4/panel/datetime-1.rc");
+        proc.shell("sed -i 's/%l/%H/' /home/demo/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+        proc.shell("sed -i 's/%l/%H/' " + skelpath + "/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-panel.xml");
+
 
         //mx kde
         proc.shell("sed -i '/use24hFormat=/c\\use24hFormat=2' /home/demo/.config/plasma-org.kde.plasma.desktop-appletsrc");
@@ -533,7 +539,7 @@ void Oobe::setUserInfo()
 
     if (QFileInfo::exists(dpath)) {
         if (gui.radioOldHomeSave->isChecked()) {
-            bool ok = false;
+            ok = false;
             QStringList cargs({"-f", dpath, dpath});
             for (int ixi = 1; ixi < 10 && !ok; ++ixi) {
                 cargs.last() = dpath + ".00" + QString::number(ixi);
@@ -603,8 +609,15 @@ void Oobe::setUserInfo()
     }
 
     // fix the ownership, demo=newuser
-    if (!proc.exec("chown", {"-R", "demo:demo", dpath})) {
-        throw QT_TR_NOOP("Sorry, failed to set ownership of user directory.");
+    ok = proc.exec("chown", {"-R", "demo:demo", dpath});
+    if (ok) {
+        // Set permissions according to /etc/adduser.conf
+        QSettings addusercfg("/etc/adduser.conf", QSettings::NativeFormat);
+        mode_t mode = addusercfg.value("DIR_MODE", "0700").toString().toUInt(&ok, 8);
+        if (ok) ok = (chmod(dpath.toUtf8().constData(), mode) == 0);
+    }
+    if (!ok) {
+        throw QT_TR_NOOP("Failed to set ownership or permissions of user directory.");
     }
 
     // change in files
@@ -612,6 +625,8 @@ void Oobe::setUserInfo()
     replaceStringInFile("demo", username, rootpath + "/etc/gshadow");
     replaceStringInFile("demo", username, rootpath + "/etc/passwd");
     replaceStringInFile("demo", username, rootpath + "/etc/shadow");
+    replaceStringInFile("demo", username, rootpath + "/etc/subuid");
+    replaceStringInFile("demo", username, rootpath + "/etc/subgid");
     replaceStringInFile("demo", username, rootpath + "/etc/slim.conf");
     replaceStringInFile("demo", username, rootpath + "/etc/lightdm/lightdm.conf");
     replaceStringInFile("demo", username, rootpath + "/home/*/.gtkrc-2.0");
