@@ -1,7 +1,12 @@
 /***************************************************************************
  * Basic partition manager for the installer.
+ ***************************************************************************
  *
- *   Copyright (C) 2019-2023 by AK-47
+ *   Copyright (C) 2019, 2020-2021 by AK-47
+ *   Transplanted code, marked with comments further down this file:
+ *    - Copyright (C) 2003-2010 by Warren Woodford
+ *    - Heavily edited, with permision, by anticapitalista for antiX 2011-2014.
+ *    - Heavily revised by dolphin oracle, adrian, and anticaptialista 2018.
  *   Licensed under the Apache License, Version 2.0 (the "License");
  *   you may not use this file except in compliance with the License.
  *   You may obtain a copy of the License at
@@ -16,6 +21,7 @@
  *
  * This file is part of the gazelle-installer.
  ***************************************************************************/
+
 #ifndef PARTMAN_H
 #define PARTMAN_H
 
@@ -27,6 +33,8 @@
 #include <QStack>
 
 #include "ui_meinstall.h"
+#include "mprocess.h"
+#include "msettings.h"
 #include "safecache.h"
 
 #define KB 1024LL
@@ -36,9 +44,6 @@
 #define PB 1125899906842624LL
 #define EB 1152921504606846976LL
 
-#define PARTMAN_SAFETY (8*MB) // 1MB at start + Compensate for rounding errors.
-#define PARTMAN_MAX_PARTS 128 // Maximum number of partitions Linux supports.
-
 class DeviceItem
 {
     Q_DECLARE_TR_FUNCTIONS(DeviceItem)
@@ -49,7 +54,10 @@ class DeviceItem
     DeviceItem *parentItem = nullptr;
     class PartMan *partman = nullptr;
     int order = -1;
+    DeviceItem *addPart(int defaultMB, const QString &defaultUse, bool crypto);
+    void driveAutoSetActive();
     void autoFill(unsigned int changed = 0xFFFF);
+    void labelParts();
 public:
     DeviceItem *active = nullptr;
     DeviceItem *origin = nullptr;
@@ -72,7 +80,7 @@ public:
         bool curESP : 1;
         bool volCrypto : 1;
     } flags = {};
-    QString model, device, path, uuid, devMapper;
+    QString model, device, path, devMapper;
     QString label, curLabel;
     QString usefor;
     QString format, curFormat;
@@ -96,15 +104,10 @@ public:
     int indexOfChild(DeviceItem *child);
     int childCount() const;
     void sortChildren();
-    // Layout finishing
-    DeviceItem *addPart(long long defaultSize, const QString &defaultUse, bool crypto);
-    void driveAutoSetActive();
-    void labelParts();
     // Helpers
     static QString realUseFor(const QString &use);
     inline QString realUseFor() const { return realUseFor(usefor); }
-    static QString shownUseFor(const QString &use);
-    inline QString shownUseFor() const { return shownUseFor(realUseFor()); }
+    QString shownUseFor() const;
     void setActive(bool boot);
     bool isActive() const;
     bool isLocked() const;
@@ -122,6 +125,7 @@ public:
     bool canMount() const;
     long long driveFreeSpace(bool inclusive = false) const;
     /* Convenience */
+    int layoutDefault(int rootPercent, bool crypto, bool updateTree=true);
     void addToCombo(QComboBox *combo, bool warnNasty = false) const;
     static QStringList split(const QString &devname);
     static QString join(const QString &drive, int partnum);
@@ -156,20 +160,17 @@ class PartMan : public QAbstractItemModel
     Q_OBJECT
     friend class DeviceItem;
     friend class DeviceItemIterator;
-    class MProcess &proc;
+    MProcess &proc;
     DeviceItem root;
     DeviceItem *changing = nullptr;
     Ui::MeInstall &gui;
+    QWidget *master;
     SafeCache key;
+    QMap<QString, QString> defaultLabels;
     bool brave, gptoverride;
     void scanVirtualDevices(bool rescan);
     void resizeColumnsToFit();
-    void preparePartitions();
-    void formatPartitions();
     void prepareSubvolumes(DeviceItem *partit);
-    bool fixCryptoSetup();
-    bool makeFstab();
-    void mountPartitions();
     void treeItemChange();
     void treeSelChange();
     void treeMenu(const QPoint &);
@@ -198,38 +199,30 @@ public:
         Pass,
         _TreeColumns_
     };
-    struct VolumeSpec
-    {
-        QString defaultLabel;
-        long long image = 0;
-        long long minimum = 0;
-        long long preferred = 0;
-    };
-    QMap<QString, struct VolumeSpec> volSpecs;
     QString bootUUID;
+    long long rootSpaceNeeded = 0;
+    long long bootSpaceNeeded = 0;
     QMap<QString, DeviceItem *> mounts;
-    class AutoPart *autopart = nullptr;
-    PartMan(class MProcess &mproc, class Ui::MeInstall &ui,
-        const class QSettings &appConf, const QCommandLineParser &appArgs);
+    PartMan(MProcess &mproc, Ui::MeInstall &ui, const QSettings &appConf, const QCommandLineParser &appArgs);
     void scan(DeviceItem *drvstart = nullptr);
-    bool manageConfig(class MSettings &config, bool save);
+    bool manageConfig(MSettings &config, bool save);
     bool composeValidate(bool automatic, const QString &project);
     bool checkTargetDrivesOK();
     DeviceItem *selectedDriveAuto();
     void clearAllUses();
     int countPrepSteps();
-    void prepStorage();
-    void installTabs();
+    void preparePartitions();
+    void formatPartitions();
     void loadKeyMaterial(const QString &keyfile);
+    bool fixCryptoSetup();
+    bool makeFstab(bool populateMediaMounts);
+    void mountPartitions();
     void unmount();
     bool willFormat(const QString &point);
     QString getMountDev(const QString &point, const bool mapped=true);
     int swapCount();
     int isEncrypt(const QString &point);
     DeviceItem *findByPath(const QString &devpath) const;
-    DeviceItem *findHostDev(const QString &path) const;
-    struct VolumeSpec volSpecTotal(const QString &path, const QStringList &vols) const;
-    inline struct VolumeSpec volSpecTotal(const QString &path) const { return volSpecTotal(path, mounts.keys()); }
     // Model View Controller
     QVariant data(const QModelIndex &index, int role) const override;
     bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
